#include <stdlib.h>
#include <iostream>

using namespace std;

class Vehicle {
   protected:
    int Number;

   public:
    Vehicle(){};                         // Default constructor
    Vehicle(int numb) : Number(numb){};  // Constructor with member initializer list
    int getNumber() const { return Number; }
};

class Car : public Vehicle {
    // friend declaration
    friend class ListOfCars;
    // Data members
   private:
    int CarNo;
    float Weight;
    Car *Successor;  // Concatenation pointer to the successor
    // Member functions
   public:
    // Constructor
    Car(int no = 0, int cNo = 0, float wei = 0.0) : Vehicle(no), CarNo(cNo), Weight(wei), Successor(0) {}
    int getCarNo() const { return CarNo; }
    float getWeight() const { return Weight; }
    void print() const { cout << "Car number: " << CarNo << ", Weight: " << Weight << endl; }
    Car *next() { return Successor; }
};

class ListOfCars  // List of cars
{
   private:
    // Head pointer
    Car *Head;

   public:
    // Constructor for an empty list
    ListOfCars() : Head(0) {}
    // Insert a Car at the beginning of the list
    void insert(Car *kp) {
        kp->Successor = Head;
        Head = kp;
        cout << "Car created." << endl;
    }

    // Delete a car at the end of the list
    Car *removeLast() {
        Car *kp = Head;
        if (Head != NULL)  // If head is not NULL (means the list is not empty)
        {
            // If there's no successor, then set the Head to NULL
            if (Head->Successor == NULL) {
                Head = NULL;
            } else {
                // Otherwise traverse to the second last element of the list
                Car *temp = Head;
                while (temp->Successor->Successor != NULL) temp = temp->Successor;

                // Change the next of the second
                // last node to null and delete the
                // last node by using free(lastNode);
                Car *lastNode = temp->Successor;
                temp->Successor = NULL;
                free(lastNode);
            }
        }
        return kp;
    }

    // Delete a car at the beginning of the list
    Car *removeFirst() {
        Car *kp = Head;
        if (kp != 0) {
            Head = kp->Successor;
            kp->Successor = 0;  // 0 is the same as NULL
            cout << "Car delete." << endl;
        }
        return kp;
    }

    Car *find(int no) {
        Car *ap;
        for (ap = begin(); ap != 0; ap = ap->next())
            if (ap->getCarNo() == no) break;
        return ap;
    }

    // Beginning of the list (Head)
    Car *begin() const { return Head; }

    // If list is empty
    bool empty() const { return Head == 0; }
};

int main()

{
    int selection = 1;
    ListOfCars list;
    int cNo, no;
    float wei;
    Car *kp;

    do {
        system("cls");  //Clears the screen for windows-systems. For unix, use system("clear"); - you can also omit it.
        cout << endl << "---- CAR MANAGEMENT ----" << endl;
        cout << "1. Create a new car" << endl;
        cout << "2. Delete car" << endl;
        cout << "3. Show list" << endl;
        cout << "4. Output data set" << endl;
        cout << "0. Exit program" << endl;
        cout << "> Make a selection [Return]: ";
        cin >> selection;
        cout << endl;

        switch (selection) {
            case 0:  // End
                cout << "Bye Bye" << endl;
                cin.ignore();
                cin.get();
                break;
            case 1:  // Create
                cout << "CarNo: ";
                cin >> cNo;
                cout << "Number of passengers: ";
                cin >> no;
                cout << "Weight: ";
                cin >> wei;
                list.insert(new Car(cNo, no, wei));
                cin.ignore();
                cin.get();
                break;
            case 2:  // Delete
                cout << "1. Delete first element" << endl;
                cout << "2. Delete last element" << endl;
                cout << "> Make a selection [Return]: " << endl;
                cin >> selection;
                if (selection == 1)
                    list.removeFirst();
                else
                    list.removeLast();
                cin.ignore();
                cin.get();
                break;
            case 3:  // List of all cars
                for (kp = list.begin(); kp != 0; kp = kp->next()) kp->print();
                cin.ignore();
                cin.get();
                break;
            case 4:  // Search for car by CarNo. and if found, then output number of passengers/weight
                Car *object;
                cout << "CarNo. to search for ";
                cin >> cNo;
                object = list.find(cNo);
                if (object == 0)
                    cout << "Car not found" << endl;
                else
                    cout << "Number of passengers: " << object->getNumber() << endl << "Weight of the car: " << object->getWeight() << endl;
                cin.ignore();
                cin.get();
                break;
            default:  // Incorrect input
                cerr << "ERROR: incorrect input" << endl;
                cin.ignore();
                cin.get();
                break;
        }
    } while (selection != 0);

    // Delete all
    while (!list.empty()) 
        list.removeFirst();

    return 0;
}
